/**
 * Copyright (c) 2023 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#pragma once
#include "display/shader.h"

extern shader_t SIMPLE_TEXTURED_SHADER;

/**
 * Initialize the simple textured shader. This is a simple shader for rendering
 * textured objects without lighting or anything fancy.
 */
void simpleTexturedShaderInit();

/**
 * Binds the simple textured shader.
 */
void simpleTexturedShaderBind();

/**
 * Sets the model matrix for the shader.
 * @param model The model matrix to set.
 */
void simpleTexturedShaderSetModel(const mat4_t model);

/**
 * Sets the color for the shader.
 * @param color The color to set.
 */
void simpleTexturedShaderSetColor(const color4f_t color);

/**
 * Sets the projection matrix for the shader.
 * @param projection The projection matrix to set.
 */
void simpleTexturedShaderSetProjection(const mat4_t projection);

/**
 * Sets the view matrix for the shader.
 * @param view The view matrix to set.
 */
void simpleTexturedShaderSetView(const mat4_t view);

/**
 * Sets the texture for the shader.
 * @param texture The texture to set.
 */
void simpleTexturedShaderSetTexture(const texture_t *texture);

/**
 * Destroys the simple textured shader.
 */
void simpleTexturedShaderDispose();